---
outline: [2,4]
---

# 在 Debian 上安装 Docker Engine

要在 Debian 上开始使用 Docker Engine，请确保你 [满足先决条件](#先决条件)，然后按照 [安装步骤](#安装方法)进行操作。

## 先决条件
### 防火墙限制
:::warning 警告
在安装 Docker 之前，请确保考虑以下安全隐患和防火墙不兼容性。
:::

- 如果你使用 ufw 或firewalld 来管理防火墙设置，请注意，当你使用 Docker 公开容器端口时，这些端口会绕过防火墙规则。有关更多信息，请参阅 [Docker 和 ufw](https://docs.docker.com/engine/network/packet-filtering-firewalls/#docker-and-ufw)。
- Docker 仅与`iptables-nft`和兼容`iptables-legacy`。使用 创建的防火墙规则`nft`在安装了 Docker 的系统上不受支持。确保你使用的任何防火墙规则集都是使用`iptables`或创建的`ip6tables`，并且将它们添加到`DOCKER-USER`链中，请参阅 [数据包过滤和防火墙](https://docs.docker.com/engine/network/packet-filtering-firewalls/)。

### 操作系统要求

要安装 Docker Engine，你需要以下 Debian 版本之一的 64 位版本：

- Debian Bookworm 12（稳定版）
- Debian Bullseye 11（旧稳定版）
Debian 的 Docker Engine 与 x86_64（或 amd64）、armhf、arm64 和 ppc64le（ppc64el）架构兼容。

### 卸载旧版本
在安装 Docker Engine 之前，你需要卸载所有有冲突的软件包。

你的 Linux 发行版可能提供了非官方的 Docker 软件包，这些软件包可能会与 Docker 提供的官方软件包冲突。你必须在安装 Docker Engine 正式版之前卸载这些软件包。

要卸载的非官方软件包包括：

- `docker.io`
- `docker-compose`
- `docker-doc`
- `podman-docker`

此外，Docker Engine 依赖于`containerd`和`runc`。Docker Engine 将这些依赖项捆绑为一个包：`containerd.io`。如果你之前安装了`containerd`或`runc`，请卸载它们以避免与 Docker Engine 捆绑的版本冲突。

运行以下命令来卸载所有冲突的包：

```bash
for pkg in docker.io docker-doc docker-compose podman-docker containerd runc; do sudo apt-get remove $pkg; done
```

`apt-get`可能会报告你没有安装这些包。

卸载 Docker 时，不会自动删除存储在中的图像、容器、卷和网络`/var/lib/docker/`。如果你想从全新安装开始，并且希望清理所有现有数据，请阅读 [卸载 Docker Engine](#卸载DockerEngine)部分。

## 安装方法

你可以通过多种方式安装 Docker 引擎，具体取决于你的需求：

- Docker Engine 与 [Docker Desktop for Linux](https://docs.docker.com/desktop/setup/install/linux/) 是捆绑在一起的。这是最简单、最快捷的入门方式。
- 从 [Docker 的 apt 仓库](https://docs.docker.com/engine/install/debian/#install-using-the-repository)中设置并安装 Docker 引擎。
- [手动安装](https://docs.docker.com/engine/install/debian/#install-from-a-package)并手动管理升级。
- 使用[简易脚本](https://docs.docker.com/engine/install/debian/#install-using-the-convenience-script)。仅推荐用于测试和开发环境。

### 使用 apt 仓库进行安装
在新主机上首次安装 Docker Engine 之前，需要设置 Docker `apt`存储库。之后，你可以从存储库安装和更新 Docker。

1. 设置 Docker 的`apt`存储库。
```bash
 # Add Docker's official GPG key:
 sudo apt-get update
 sudo apt-get install ca-certificates curl
 sudo install -m 0755 -d /etc/apt/keyrings
 sudo curl -fsSL https://download.docker.com/linux/debian/gpg -o /etc/apt/keyrings/docker.asc
 sudo chmod a+r /etc/apt/keyrings/docker.asc

 # Add the repository to Apt sources:
 echo \
   "deb [arch=$(dpkg --print-architecture) signed-by=/etc/apt/keyrings/docker.asc] https://download.docker.com/linux/debian \
   $(. /etc/os-release && echo "$VERSION_CODENAME") stable" | \
   sudo tee /etc/apt/sources.list.d/docker.list > /dev/null
 sudo apt-get update
```
:::info 笔记
如果你使用衍生发行版，例如 Kali Linux，你可能需要替换此命令中预期打印版本代号的部分：
```bash
$(. /etc/os-release && echo "$VERSION_CODENAME")
```
将这部分替换为相应 Debian 版本的代号，例如`bookworm` 。
:::
2. 安装 Docker 软件包。
:::code-group
``` bash [最新的]
# 安装 Docker 软件包。
sudo apt-get install docker-ce docker-ce-cli containerd.io docker-buildx-plugin docker-compose-plugin
```
```bash [具体版本]
# 要安装特定版本的 Docker Engine，请首先列出存储库中的可用版本：
# List the available versions:
apt-cache madison docker-ce | awk '{ print $3 }'

5:27.4.0-1~debian.12~bookworm
5:27.3.1-1~debian.12~bookworm
...

# 或 

# 选择所需的版本并安装：
VERSION_STRING=5:27.4.0-1~debian.12~bookworm
sudo apt-get install docker-ce=$VERSION_STRING docker-ce-cli=$VERSION_STRING containerd.io docker-buildx-plugin docker-compose-plugin
```
:::

3. 通过运行`hello-world`映像验证安装是否成功：
```bash 
sudo docker run hello-world
```
此命令下载测试映像并在容器中运行它。当容器运行时，它会打印一条确认消息并退出。

你现在已经成功安装并启动了 Docker Engine。
:::tip 提示
尝试在没有 root 的情况下运行时收到错误？
`docker`用户组存在但不包含任何用户，这就是为什么你需要使用`sudo`来运行 Docker 命令。继续 [Linux 安装后](https://docs.docker.com/engine/install/linux-postinstall) 允许非特权用户运行 Docker 命令以及其他可选配置步骤。
:::

#### 升级 Docker 引擎
要升级 Docker Engine，请按照以上步骤 2 进行操作： [安装说明](#安装方法)，选择你要安装的新版本。

### 从包安装
如果你无法使用 Docker 的`apt`存储库来安装 Docker Engine，你可以下载适合你的版本的`deb`文件并手动安装。每次升级 Docker Engine 时都需要下载新文件。

1. 前往 [https://download.docker.com/linux/debian/dists/](https://download.docker.com/linux/debian/dists/).
2. 在列表中选择你的 Debian 版本。
3. 转到`pool/stable`/并选择适用的架构（ `amd64` 、 `armhf` 、 `arm64`或`s390x` ）。
4. 下载 Docker 引擎、CLI、containerd 和 Docker Compose 软件包的以下`deb`文件：
   - `containerd.io_<version>_<arch>.deb`
   - `docker-ce_<version>_<arch>.deb`
   - `docker-ce-cli_<version>_<arch>.deb`
   - `docker-buildx-plugin_<version>_<arch>.deb`
   - `docker-compose-plugin_<version>_<arch>.deb`
5. 安装`.deb`软件包。将以下示例中的路径更新为下载 Docker 软件包的位置。
```bash
sudo dpkg -i ./containerd.io_<version>_<arch>.deb \
  ./docker-ce_<version>_<arch>.deb \
  ./docker-ce-cli_<version>_<arch>.deb \
  ./docker-buildx-plugin_<version>_<arch>.deb \
  ./docker-compose-plugin_<version>_<arch>.deb
```
Docker 守护进程自动启动。
6. 通过运行`hello-world`映像验证安装是否成功：
```bash
sudo service docker start
sudo docker run hello-world
```
此命令下载测试映像并在容器中运行它。当容器运行时，它会打印一条确认消息并退出。

你现在已经成功安装并启动了 Docker Engine。

:::tip 提示
尝试在没有 root 的情况下运行时收到错误？

docker用户组存在但不包含任何用户，这就是为什么你需要使用sudo来运行 Docker 命令。继续 [Linux 安装后](https://docs.docker.com/engine/install/linux-postinstall) 允许非特权用户运行 Docker 命令以及其他可选配置步骤。
:::

#### 升级 Docker 引擎
要升级 Docker Engine，请下载较新的软件包文件并重复 [安装过程](#从包安装)，指向新文件。

### 使用便捷脚本安装
Docker 提供了一个方便的脚本，位于 [https://get.docker.com/](https://get.docker.com/)将 Docker 安装到 非交互的开发环境。便利脚本不是 推荐用于生产环境，但它对于创建 根据你的需求定制配置脚本。另请参阅 [使用存储库安装](https://docs.docker.com/engine/install/debian/#install-using-the-repository)步骤学习 有关使用包存储库进行安装的安装步骤。源代码 因为该脚本是开源的，你可以在 [GitHub 上的`docker-install`存储库](https://github.com/docker/docker-install)。

在本地运行脚本之前，请务必先检查从互联网下载的脚本。安装之前，请熟悉便利脚本的潜在风险和限制：
- 该脚本需要`root`或`sudo`权限才能运行。
- 该脚本尝试检测你的 Linux 发行版和版本，并为你配置包管理系统。
- 该脚本不允许你自定义大多数安装参数。
- 该脚本会安装依赖项和建议，而不要求确认。这可能会安装大量软件包，具体取决于主机的当前配置。
- 默认情况下，该脚本会安装最新稳定版本的 Docker、containerd 和 runc。使用此脚本配置计算机时，可能会导致 Docker 主要版本意外升级。在部署到生产系统之前，始终在测试环境中测试升级。
- 该脚本并非旨在升级现有的 Docker 安装。使用脚本更新现有安装时，依赖项可能无法更新到预期版本，从而导致版本过时。
:::tip 提示
运行前预览脚本步骤。你可以使用`--dry-run`选项运行脚本来了解脚本在调用时将运行哪些步骤：
```bash
curl -fsSL https://get.docker.com -o get-docker.sh
sudo sh ./get-docker.sh --dry-run
```
:::

此示例从以下位置下载脚本 [https://get.docker.com/](https://get.docker.com/)并运行它以在 Linux 上安装 Docker 的最新稳定版本：
```bash
curl -fsSL https://get.docker.com -o get-docker.sh
sudo sh get-docker.sh
Executing docker install script, commit: 7cae5f8b0decc17d6571f9f52eb840fbc13b2737
<...>
```

你现在已经成功安装并启动了 Docker Engine。 `docker` 服务在基于 Debian 的发行版上自动启动。
在基于`RPM`的发行版上，例如 CentOS、Fedora、RHEL 或 SLES，你需要使用适当的`systemctl`或`service`命令手动启动它。
正如消息所示，默认情况下非 root 用户无法运行 Docker 命令。


:::info 以非特权用户身份使用 Docker，还是以无根模式安装？
安装脚本需要`roo`t或`sudo`权限才能安装 使用 Docker。
如果你想授予非root用户访问Docker的权限，请参阅 [Linux 的安装后步骤](https://docs.docker.com/engine/install/linux-postinstall/#manage-docker-as-a-non-root-user)。
你还可以在没有`root`权限的情况下安装 Docker，或者配置为运行在 无根模式。
有关以无根模式运行 Docker 的说明，请参阅 [以非 root 用户身份运行 Docker 守护进程（无 root 模式）](https://docs.docker.com/engine/security/rootless/) 。
:::

#### 安装预发行版
Docker 还提供了一个方便的脚本： [https://test.docker.com/](https://test.docker.com/)在 Linux 上安装 Docker 的预发行版。
此脚本与`get.docker.com`上的脚本相同，但将包管理器配置为使用 Docker 包存储库的测试通道。
测试通道包括 Docker 的稳定版和预发行版（测试版、候选版）。
使用此脚本可以尽早访问新版本，并在稳定版本发布之前在测试环境中对其进行评估。

要从测试通道在 Linux 上安装最新版本的 Docker，请运行：

```bash
curl -fsSL https://test.docker.com -o test-docker.sh
sudo sh test-docker.sh
```

#### 使用便捷脚本后升级 Docker
如果你使用便捷脚本安装了 Docker，则应直接使用包管理器升级 Docker。
重新运行便利脚本没有任何好处。如果它尝试重新安装主机上已存在的存储库，重新运行它可能会导致问题。

## 卸载 Docker 引擎
1. 卸载 Docker Engine、CLI、containerd 和 Docker Compose 软件包：
```bash
sudo apt-get purge docker-ce docker-ce-cli containerd.io docker-buildx-plugin docker-compose-plugin docker-ce-rootless-extras
```
2. 主机上的映像、容器、卷或自定义配置文件不会自动删除。要删除所有映像、容器和卷：
```bash
sudo rm -rf /var/lib/docker
sudo rm -rf /var/lib/containerd
```
3. 删除源列表和密钥环
```bash
sudo rm /etc/apt/sources.list.d/docker.list
sudo rm /etc/apt/keyrings/docker.asc
```
你必须手动删除任何编辑的配置文件。

## 后续步骤
继续 [Linux 的安装后步骤](https://docs.docker.com/engine/install/linux-postinstall/)。